package cn.fw.valhalla.common.utils;

import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.data.base.db.query.Page;
import cn.fw.data.base.domain.common.Message;
import org.apache.commons.collections4.CollectionUtils;

import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author huang.guo
 * @since 2021-03-20
 */
public class PageUtils {

    /**
     * page转换
     *
     * @param originalPage  原page
     * @param dataConverter 数据转换器
     * @param <O>
     * @param <T>
     * @return
     */
    public static <O, T> PageData<T> convertPage(PageData<O> originalPage, Function<O, T> dataConverter) {
        PageData<T> resultPage = new PageData<T>((int) originalPage.getCurrent(), (int) originalPage.getSize());
        resultPage.setTotal(originalPage.getTotal()).setPages(originalPage.getPages());
        if (CollectionUtils.isEmpty(originalPage.getRecords())) {
            return resultPage;
        }

        resultPage.setRecords(originalPage.getRecords()
                .stream().map(dataConverter).collect(Collectors.toList()));
        return resultPage;
    }

    /**
     * page转换
     *
     * @param originalPage
     * @param listConverter
     * @param <O>
     * @param <T>
     * @return
     */
    public static <O, T> PageData<T> convertPages(PageData<O> originalPage, Function<List<O>, List<T>> listConverter) {
        PageData<T> resultPage = new PageData<T>((int) originalPage.getCurrent(), (int) originalPage.getSize());
        resultPage.setTotal(originalPage.getTotal()).setPages(originalPage.getPages());
        if (CollectionUtils.isEmpty(originalPage.getRecords())) {
            return resultPage;
        }

        resultPage.setRecords(listConverter.apply(originalPage.getRecords()));
        return resultPage;
    }

    @SuppressWarnings("unchecked")
    public static <T> Message<Page<T>> defaultPage() {
        return Message.success(new Page<>());
    }
}
