package cn.fw.valhalla.rpc.cas;

import cn.fw.cas.sdk.api.CasApiService;
import cn.fw.data.base.domain.common.Message;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.Objects;

/**
 * @author : kurisu
 * @className : CasRpcService
 * @description : 售后rpc接口
 * @date: 2020-08-12 17:34
 */
@Slf4j
@Service
public class CasRpcService {
    private final CasApiService casApiService;

    public CasRpcService(final CasApiService casApiService) {
        this.casApiService = casApiService;
    }


    public int getPullinTimes(final Long groupId, final Long customerId) {
        try {
            Message<Integer> msg = casApiService.queryPullinTimes(groupId, customerId);
            log.info("调用售后查询进站次数接口状态: {}", msg.isSuccess());
            if (!msg.isSuccess()) {
                return 0;
            }
            Integer data = msg.getData();
            log.info("查询进站次数 data[{}]", data);
            if (Objects.isNull(data)) {
                return 0;
            }
            return data;
        } catch (Exception e) {
            log.error("查询进站次数失败", e);
            return 0;
        }
    }
}
