package cn.fw.valhalla.controller.app;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.IgnoreAuth;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.dto.FollowAttachmentDTO;
import cn.fw.valhalla.domain.vo.follow.FollowDetailVO;
import cn.fw.valhalla.domain.vo.follow.FollowRecordVO;
import cn.fw.valhalla.service.bus.follow.PCFollowBizService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import javax.validation.constraints.NotNull;
import java.util.List;

import static cn.fw.common.businessvalidator.Validator.BV;
import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * @author : kurisu
 * @className : FollowController
 * @description : 跟进控制器
 * @date: 2020-08-15 16:23
 */
@Slf4j
@RestController
@Authorization(AuthType.APP)
@Validated
@RequestMapping("/app/public/clue")
public class PublicClueFollowController {
    private final PCFollowBizService pcFollowBizService;

    @Autowired
    public PublicClueFollowController(final PCFollowBizService pcFollowBizService) {
        this.pcFollowBizService = pcFollowBizService;
    }

    @GetMapping("/detail")
    @IgnoreAuth
    @ControllerMethod("公共线索池详情")
    public Message<FollowDetailVO> detail(@NotNull(message = "id不能为空") final Long id) {
        return success(pcFollowBizService.getDetail(id));
    }

    @GetMapping("/todo/history")
    @IgnoreAuth
    @ControllerMethod("公共线索池跟进历史记录")
    public Message<List<FollowRecordVO>> history(@NotNull(message = "id不能为空") final Long id) {
        return success(pcFollowBizService.getRecordList(id));
    }

    @PostMapping("/todo/upload")
    @IgnoreAuth
    @ControllerMethod("上传跟进凭证")
    public Message<Void> upload(@CurrentUser LoginAuthBean currentUser, @RequestBody @Valid final FollowAttachmentDTO dto) {
        BV.notNull(dto.getFollowType(), "跟进类型不正确");
        pcFollowBizService.uploadAtt(dto, currentUser.getUserId());
        return success();
    }

    @GetMapping("/call")
    @IgnoreAuth
    @ControllerMethod("公共线索池拨打电话")
    public Message<String> call(@NotNull(message = "id不能为空") final Long id) {
        return success(pcFollowBizService.call(id));
    }

}
