package cn.fw.valhalla.controller.wx;

import cn.fw.common.web.annotation.ControllerMethod;
import cn.fw.common.web.auth.PassportAuthBean;
import cn.fw.common.web.auth.annotation.CurrentUser;
import cn.fw.data.base.domain.common.Message;
import cn.fw.security.auth.client.annotation.Authorization;
import cn.fw.security.auth.client.annotation.RepeatSubmit;
import cn.fw.security.auth.client.enums.AuthType;
import cn.fw.valhalla.domain.dto.*;
import cn.fw.valhalla.domain.vo.customer.*;
import cn.fw.valhalla.service.bus.cust.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import java.util.List;

import static cn.fw.common.web.util.ResultBuilder.success;

/**
 * 查询是否
 *
 * @author : xiatian
 * @date: 2022-06-06 15:00
 */
@Slf4j
@Validated
@RestController
@RequiredArgsConstructor
@Authorization(AuthType.WECHAT)
@RequestMapping("/wx/vehicle")
public class VehicleWxController {

    private final MemberVehicleBizService memberVehicleBizService;

    /**
     * 查询认证车辆
     *
     * @param user
     * @return
     */
    @GetMapping("/list")
    @ControllerMethod("查询所有认证车辆")
    public Message<List<MemberVehicleVO>> vehicleList(@CurrentUser PassportAuthBean user) {
        return success(memberVehicleBizService.getVehicleList(user.getUserId()));
    }

    /**
     * 新增变更档案
     *
     * @param user
     * @param dto
     * @return
     */
    @RepeatSubmit
    @PostMapping("/save")
    @ControllerMethod("新增认证车辆")
    public Message<Void> saveVehicle(@CurrentUser PassportAuthBean user,
                                     @Validated @RequestBody final NewVehicleDTO dto) {
        dto.setMemberId(user.getUserId());
        memberVehicleBizService.saveVehicle(dto);
        return success();
    }

}
