package cn.fw.valhalla.service.bus;

import cn.fw.common.data.mybatis.pagination.PageData;
import cn.fw.common.page.AppPage;
import cn.fw.common.web.annotation.DisLock;
import cn.fw.common.web.auth.LoginAuthBean;
import cn.fw.data.base.domain.common.Message;
import cn.fw.hermes.sdk.api.ImSendMessage;
import cn.fw.hermes.sdk.api.para.MsgParamCondition;
import cn.fw.valhalla.common.constant.RoleCode;
import cn.fw.valhalla.common.utils.DateUtil;
import cn.fw.valhalla.domain.db.LeaveNeedDo;
import cn.fw.valhalla.domain.db.customer.Customer;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.db.follow.FollowClue;
import cn.fw.valhalla.domain.db.pool.StammkundePool;
import cn.fw.valhalla.domain.dto.DistributableDTO;
import cn.fw.valhalla.domain.dto.LeaveAllocationDTO;
import cn.fw.valhalla.domain.dto.RoleChangeDTO;
import cn.fw.valhalla.domain.dto.StammkundeDto;
import cn.fw.valhalla.domain.enums.*;
import cn.fw.valhalla.domain.query.LeaveQueryVO;
import cn.fw.valhalla.domain.vo.DistributableVO;
import cn.fw.valhalla.domain.vo.LeaveNeedDoVO;
import cn.fw.valhalla.rpc.backlog.TodoRpcService;
import cn.fw.valhalla.rpc.backlog.dto.BackLogItemDTO;
import cn.fw.valhalla.rpc.ehr.EhrRpcService;
import cn.fw.valhalla.rpc.ehr.dto.StaffInfoDTO;
import cn.fw.valhalla.rpc.erp.UserService;
import cn.fw.valhalla.rpc.erp.dto.PostUserDTO;
import cn.fw.valhalla.rpc.erp.dto.UserInfoDTO;
import cn.fw.valhalla.rpc.erp.dto.UserRoleDataRangeDTO;
import cn.fw.valhalla.rpc.oop.OopService;
import cn.fw.valhalla.rpc.oop.dto.ShopDTO;
import cn.fw.valhalla.service.bus.follow.FollowBizService;
import cn.fw.valhalla.service.data.*;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.BoundListOperations;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.stream.Collectors;

import static cn.fw.common.businessvalidator.Validator.BV;

/**
 * @author : kurisu
 * @className : LeaveNeedDoBizService
 * @description : 人员变动处理
 * @date: 2020-10-16 17:04
 */
@Service
@Slf4j
@RequiredArgsConstructor
public class LeaveNeedDoBizService {
    private final LeaveNeedDoService leaveNeedDoService;
    private final CustomerService customerService;
    private final UserService userService;
    private final EhrRpcService ehrRpcService;
    private final OopService oopService;
    private final FollowBizService followBizService;
    private final ImSendMessage imSendMessage;
    private final TodoRpcService todoRpcService;
    private final StammkundePoolService stammkundePoolService;
    private final FollowClueService followClueService;
    private final ClueTaskService clueTaskService;

    /**
     * Redis工具
     */
    private final StringRedisTemplate redisTemplate;

    @Value("${spring.cache.custom.global-prefix}:LeaveNeedDo")
    @Getter
    private String keyPrefix;

    @Value("${follow.todo.leave2do}")
    @Getter
    private String leave2doCode;

    @Value("${spring.cache.custom.global-prefix}:role-change:need-end-task")
    @Getter
    private String needDealTaskKey;


    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getKeyPrefix()", key = "#shopId + ':add:' + #userId", message = "请勿重复提交")
    public void add(final Long userId, final Long shopId) {
        List<PostUserDTO> postUserDTOS = userService.getUserByRole(shopId, RoleCode.BYKFP);
        BV.isFalse(CollectionUtils.isEmpty(postUserDTOS), () -> "该门店没有配置保有客分配人员");
        LeaveNeedDo db = addable(userId, shopId);
        leaveNeedDoService.save(db);
        push2Todo(db.getId(), postUserDTOS.get(0).getUserId(), shopId);
    }

    @Transactional(rollbackFor = Exception.class)
    public void addLeaveTodo(LeaveNeedDo leaveNeedDo) {
        Long userId = leaveNeedDo.getUserId();
        Long shopId = leaveNeedDo.getShopId();
        List<PostUserDTO> postUserDTOS = userService.getUserByRole(shopId, RoleCode.BYKFP);
        BV.isFalse(CollectionUtils.isEmpty(postUserDTOS), () -> "该门店没有配置保有客分配人员");
        boolean bool = leaveNeedDoService.count(Wrappers.<LeaveNeedDo>lambdaQuery()
                .eq(LeaveNeedDo::getUserId, userId)
                .eq(LeaveNeedDo::getShopId, shopId)
                .eq(LeaveNeedDo::getDone, Boolean.FALSE)
        ) > 0;
        if (bool) {
            return;
        }
        int count = customerService.count(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getAdviserId, userId)
                .eq(Customer::getShopId, shopId)
                .eq(Customer::getYn, Boolean.TRUE)
        );
        if (count <= 0) {
            return;
        }
        leaveNeedDoService.save(leaveNeedDo);
        Map<String, String> map = new HashMap<>();
        map.put("id", String.valueOf(leaveNeedDo.getId()));
        map.put("shopId", String.valueOf(shopId));
        map.put("userName", leaveNeedDo.getUserName());
        map.put("customerNum", String.valueOf(count));
        map.put("leaveTime", DateUtil.getFullDateString(leaveNeedDo.getEffectiveTime(), "-"));
        push2Todo(leaveNeedDo.getId(), postUserDTOS.get(0).getUserId(), shopId, map);
    }

    public AppPage<LeaveNeedDoVO> getList(LoginAuthBean currentUser, LeaveQueryVO queryVO) {
        List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(currentUser.getUserId(), RoleCode.BYKFP);
        List<Long> shopIds = dataRange.stream().map(UserRoleDataRangeDTO::getRangeValue).collect(Collectors.toList());
        BV.isFalse(CollectionUtils.isEmpty(shopIds), () -> "无权限操作，请检查角色权限是否正确");
        PageData<LeaveNeedDo> pageData = leaveNeedDoService.page(new PageData<>(queryVO), Wrappers.<LeaveNeedDo>lambdaQuery()
                .in(LeaveNeedDo::getShopId, shopIds)
                .eq(LeaveNeedDo::getDone, Boolean.FALSE)
                .eq(Objects.nonNull(queryVO.getType()), LeaveNeedDo::getType, queryVO.getType())
        );
        AppPage<LeaveNeedDoVO> page = AppPage.empty(queryVO);
        List<LeaveNeedDoVO> list = new ArrayList<>();
        List<LeaveNeedDo> records = Optional.ofNullable(pageData.getRecords()).orElse(new ArrayList<>());
        for (LeaveNeedDo needDo : records) {
            int i = customerService.count(Wrappers.<Customer>lambdaQuery()
                    .eq(Customer::getAdviserId, needDo.getUserId())
                    .eq(Customer::getYn, Boolean.TRUE)
            );
            list.add(LeaveNeedDoVO.with(needDo, i));
        }
        page.setData(list);
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getKeyPrefix()", key = "#dto.id", message = "请勿重复提交")
    public void allocation(LoginAuthBean user, LeaveAllocationDTO dto) {
        final String key = generateKey(dto.getId());
        try {
            //准备分配 常规校验
            prepareAllocation(dto);
            //变更档案
            doAllocation(dto, key);
            //完成分配发送消息提醒等
            finish(user, dto.getId(), key);
        } finally {
            clearKey(key);
        }
    }

    /**
     * 服务顾问角色变动跟进处理
     *
     * @param leaveId
     */
    @Transactional(rollbackFor = Exception.class)
    public void fwgwChanged(Long leaveId) {
        LeaveNeedDo leaveNeedDo = leaveNeedDoService.getById(leaveId);
        if (Objects.isNull(leaveNeedDo)) {
            return;
        }
        List<ClueTask> list = clueTaskService.list(Wrappers.<ClueTask>lambdaQuery()
                .in(ClueTask::getType, FollowTypeEnum.FM, FollowTypeEnum.RM)
                .eq(ClueTask::getState, TaskStateEnum.ONGOING)
                .eq(ClueTask::getFollowUser, leaveNeedDo.getUserId())
        );
        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        for (ClueTask task : list) {
            followBizService.roleChangeEndTask(task);
        }
    }

    /**
     * 续保角色变动
     *
     * @param dto
     */
    @Transactional(rollbackFor = Exception.class)
    public void xbgjChanged(RoleChangeDTO dto) {
        final Long userId = dto.getUserId();
        List<ClueTask> list = clueTaskService.list(Wrappers.<ClueTask>lambdaQuery()
                .eq(ClueTask::getType, FollowTypeEnum.IR)
                .eq(ClueTask::getState, TaskStateEnum.ONGOING)
                .eq(ClueTask::getFollowUser, userId)
        );

        if (CollectionUtils.isEmpty(list)) {
            return;
        }

        for (ClueTask task : list) {
            followBizService.roleChangeEndTask(task);
        }
    }

    public List<DistributableVO> distributableList(Long shopId) {
        List<DistributableVO> list = new ArrayList<>();
        List<Customer> customerList = distributableCustomer(shopId);
        if (CollectionUtils.isEmpty(customerList)) {
            return list;
        }
        Map<Long, List<Customer>> userCustomer = customerList.stream().collect(Collectors.groupingBy(Customer::getAdviserId));
        for (Map.Entry<Long, List<Customer>> entry : userCustomer.entrySet()) {
            DistributableVO vo = new DistributableVO();
            StaffInfoDTO staffInfoDTO = ehrRpcService.queryStaffInfo(entry.getKey());
            vo.setUserId(entry.getKey());
            vo.setUserName(Optional.ofNullable(staffInfoDTO).map(StaffInfoDTO::getName).orElse(""));
            vo.setDistributableNum(entry.getValue().size());
            vo.setShopId(shopId);
            list.add(vo);
        }
        return list;
    }


    @Transactional(rollbackFor = Exception.class)
    @DisLock(prefix = "#this.getKeyPrefix()", key = "'shop:' + #dto.shopId", message = "请勿重复提交")
    public void activeAllocation(LoginAuthBean user, DistributableDTO dto) {
        //准备分配 常规校验
        Map<Long, List<Customer>> map = verifyAndPrepare(dto);
        //变更档案
        List<Customer> list = activeDoAllocation(dto, map);
        int size = list.size();
        if (size == 0) {
            return;
        }
        try {
            String text = String.format("%s通过资源分配给你%s台保有客", user.getUserName(), size);
            final MsgParamCondition msgPara = MsgParamCondition.getOfflineTxetPara(text, null, "保有客分配",
                    "保有客分配", null, dto.getAdviserId()).build();
            final Message<Boolean> msg = imSendMessage.synSendMsg(msgPara);
            log.info("给[{}]推送im消息结果:[{}]", dto.getAdviserName(), msg.getResult());
        } catch (Exception e) {
            log.error("给[{}]推送im消息失败：]", dto.getAdviserName(), e);
        }
    }

    private void prepareAllocation(LeaveAllocationDTO dto) {
        LeaveNeedDo needDo = leaveNeedDoService.queryProcessableById(dto.getId());
        BV.notNull(needDo, () -> "该条记录已处理或不存在，请刷新后重试");
        dto.setAdviserId(needDo.getUserId());

        List<LeaveAllocationDTO.Allocation> ruleList = dto.getRuleList();
        BV.notNull(ruleList, () -> "分配规则不能为空");

        for (LeaveAllocationDTO.Allocation rule : ruleList) {
            List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(rule.getUserId(), RoleCode.FWGW);
            List<Long> shopIdList = dataRange.stream().map(UserRoleDataRangeDTO::getRangeValue).collect(Collectors.toList());
            BV.isNotEmpty(shopIdList, () -> String.format("[%s]无服务接待角色，请核对", rule.getUserName()));
            Long shopId = shopIdList.get(0);
            BV.isTrue(needDo.getShopId().equals(shopId), () -> String.format("[%s]所属门店与离职人员门店不符", rule.getUserName()));
            rule.setShopId(shopId);
        }
    }

    /**
     * 处理数据
     *
     * @param dto
     * @return 处理完的档案
     */
    private void doAllocation(LeaveAllocationDTO dto, String key) {
        List<Customer> customerList = customerService.queryByAdviserId(dto.getAdviserId());
        if (CollectionUtils.isEmpty(customerList)) {
            return;
        }
        List<LeaveAllocationDTO.Allocation> ruleList = dto.getRuleList();

        final List<Customer> list = new ArrayList<>();
        final List<StammkundePool> spl = new ArrayList<>();
        final LinkedList<UserInfo> queue = new LinkedList<>();

        for (LeaveAllocationDTO.Allocation rule : ruleList) {
            int size = customerList.size();
            List<Customer> subList = customerList.subList(0, rule.getNum() > size ? size : rule.getNum());
            if (CollectionUtils.isEmpty(subList)) {
                continue;
            }
            for (Customer customer : subList) {
                customer.setShopId(rule.getShopId());
                customer.setAdviserId(rule.getUserId());
                rejectPool(customer.getId(), rule.getShopId(), rule.getUserName(), rule.getUserId(), customer.getGroupId());
                spl.add(createPool(customer, rule.getUserName()));
                list.add(customer);
            }
            queue.add(new UserInfo(rule.getUserId(), rule.getUserName(), subList.size()));
            subList.clear();
        }
        customerService.updateBatchById(list);
        stammkundePoolService.saveBatch(spl);
        setToCache(key, queue);
    }

    private void finish(LoginAuthBean user, Long leaveId, String key) {
        leaveNeedDoService.dealById(leaveId);
        List<UserInfo> list = getAllFromCache(key);
        for (UserInfo info : list) {
            try {
                String text = String.format("%s通过资源分配给你%s台保有客", user.getUserName(), info.getCount());
                final MsgParamCondition msgPara = MsgParamCondition.getOfflineTxetPara(text, null, "保有客分配",
                        "保有客分配", null, info.getUserId()).build();
                final Message<Boolean> msg = imSendMessage.synSendMsg(msgPara);
                log.info("给[{}]推送im消息结果:[{}]", info.getUserName(), msg.getResult());
            } catch (Exception e) {
                log.error("给[{}]推送im消息失败：", info, e);
            }
        }
        clearKey(key);
        BackLogItemDTO backLogItemDTO = new BackLogItemDTO(user.getUserId(), getLeave2doCode(), String.valueOf(leaveId), new Date(), null);
        todoRpcService.complete(backLogItemDTO);

        BoundListOperations<String, String> listOps = redisTemplate.boundListOps(getNeedDealTaskKey());
        listOps.rightPush(String.valueOf(leaveId));
    }

    private void rejectPool(Long customerId, Long shopId, String userName, Long userId, Long groupId) {
        StammkundeDto stammkundeDto = new StammkundeDto();
        stammkundeDto.setCustomerId(customerId);
        stammkundeDto.setReason(DefeatReasonEnum.LC);
        stammkundeDto.setShopId(shopId);
        stammkundeDto.setNewUserId(userId);
        stammkundeDto.setNewUserName(userName);
        ShopDTO shop = oopService.shop(shopId);
        stammkundeDto.setNewShopId(shopId);
        if (Objects.nonNull(shop)) {
            stammkundeDto.setNewShopName(shop.getShortName());
        }
        stammkundeDto.setGroupId(groupId);

        stammkundePoolService.reject(stammkundeDto);
    }

    private StammkundePool createPool(Customer customer, String userName) {
        StammkundePool sp = new StammkundePool();
        sp.setCustomerId(customer.getId());
        sp.setAdviserId(customer.getAdviserId());
        sp.setAdviserName(userName);
        sp.setSources(StammkundeSourcesEnum.SYSTEMATIC);
        sp.setPoolStatus(StammkundeStatusEnum.KUNDE);
        sp.setAktiv(Boolean.TRUE);
        sp.setActivationTime(new Date());
        sp.setShopId(customer.getShopId());
        sp.setGroupId(customer.getGroupId());
        return sp;
    }

    private String generateKey(final Long leaveId) {
        Assert.notNull(leaveId, "leaveId cannot be null");

        return String.format("%s:allocation:%s", getKeyPrefix(), leaveId);
    }

    public List<UserInfo> getAllFromCache(final String key) {
        try {
            Boolean hasKey = redisTemplate.hasKey(key);
            if (!Boolean.TRUE.equals(hasKey)) {
                return new ArrayList<>();
            }
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            List<String> stringList = ops.range(0, -1);
            if (CollectionUtils.isEmpty(stringList)) {
                return new ArrayList<>();
            }
            List<UserInfo> dtos = stringList.stream().map(str -> JSONObject.parseObject(str, UserInfo.class)).collect(Collectors.toList());
            redisTemplate.delete(key);
            return dtos;
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
            return new ArrayList<>();
        }
    }


    private void setToCache(final String key, final LinkedList<UserInfo> userInfos) {
        try {
            if (CollectionUtils.isEmpty(userInfos)) {
                return;
            }
            BoundListOperations<String, String> ops = redisTemplate.boundListOps(key);
            String[] users = userInfos.stream().map(JSONObject::toJSONString).toArray(String[]::new);
            ops.rightPushAll(users);
        } catch (Exception e) {
            log.error("缓存设置信息失败 key[{}]", key, e);
        }
    }

    private void clearKey(final String key) {
        try {
            redisTemplate.delete(key);
        } catch (Exception e) {
            log.error("清空缓存信息失败 key[{}]", key, e);
        }
    }

    private LeaveNeedDo addable(final Long userId, final Long shopId) {
        List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(userId, RoleCode.FWGW);
        boolean ok = CollectionUtils.isEmpty(dataRange) || !Objects.equals(dataRange.get(0).getRangeValue(), shopId);
        BV.isTrue(ok, () -> "请先移除对对应服务站服务顾问角色后操作");
        int count = leaveNeedDoService.count(Wrappers.<LeaveNeedDo>lambdaQuery()
                .eq(LeaveNeedDo::getUserId, userId)
                .eq(LeaveNeedDo::getShopId, shopId)
                .eq(LeaveNeedDo::getDone, Boolean.FALSE)
        );
        BV.isTrue(count == 0, () -> "已存在待分配记录，请勿重复添加");
        UserInfoDTO user = ehrRpcService.user(userId);
        BV.notNull(user, () -> "用户不存在");
        boolean bool = customerService.count(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getAdviserId, userId)
                .eq(Customer::getShopId, shopId)
                .eq(Customer::getYn, Boolean.TRUE)
        ) > 0;
        BV.isTrue(bool, () -> "该顾问没有可用档案无需分配保有客");
        LeaveNeedDo db = new LeaveNeedDo();
        db.setDone(Boolean.FALSE);
        db.setEffectiveTime(DateUtil.getMonthEndDay(new Date()));
        db.setReason(LeaveReasonEnum.OTHER);
        db.setType(LeaveTodoTypeEnum.CUSTOMER);
        db.setShopId(shopId);
        db.setUserId(userId);
        db.setUserName(user.getUserName());
        db.setCreateTime(new Date());
        db.setUpdateTime(new Date());
        return db;
    }

    private void push2Todo(Long id, Long userId, Long shopId) {
        Map<String, String> map = new HashMap<>();
        map.put("shopId", String.valueOf(shopId));
        map.put("id", String.valueOf(id));
        push2Todo(id, userId, shopId, map);
    }

    private void push2Todo(Long id, Long userId, Long shopId, Map<String, String> map) {
        BackLogItemDTO dto = new BackLogItemDTO(userId, getLeave2doCode(), String.valueOf(id), new Date(), shopId);
        dto.setDynamicMap(map);
        todoRpcService.push(dto);
    }

    private List<Customer> distributableCustomer(Long shopId) {
        Set<String> vinSet = new HashSet<>();
        List<FollowClue> clueList = followClueService.list(Wrappers.<FollowClue>lambdaQuery()
                .eq(FollowClue::getClueState, ClueStatusEnum.ONGOING)
                .ne(FollowClue::getClueType, FollowTypeEnum.AC)
        );

        if (!CollectionUtils.isEmpty(clueList)) {
            vinSet.addAll(clueList.stream().map(FollowClue::getVin).collect(Collectors.toSet()));
        }

        return customerService.list(Wrappers.<Customer>lambdaQuery()
                .eq(Customer::getYn, Boolean.TRUE)
                .eq(Customer::getShopId, shopId)
                .notIn(!CollectionUtils.isEmpty(vinSet), Customer::getFrameNo, vinSet)
                .isNull(Customer::getTags)
                .isNull(Customer::getCusLevel)
        );
    }

    private Map<Long, List<Customer>> verifyAndPrepare(DistributableDTO dto) {
        List<UserRoleDataRangeDTO> dataRange = userService.getUserRoleDataRange(dto.getAdviserId(), RoleCode.FWGW);
        List<Long> shopIdList = dataRange.stream().map(UserRoleDataRangeDTO::getRangeValue).collect(Collectors.toList());
        BV.isNotEmpty(shopIdList, () -> String.format("[%s]无服务接待角色，请核对", dto.getAdviserName()));

        List<DistributableDTO.BeAssignedDTO> beAssignedList = dto.getBeAssignedList();
        BV.isNotEmpty(beAssignedList, () -> "分配规则不能为空");

        List<Customer> customerList = distributableCustomer(dto.getShopId());
        BV.isNotEmpty(customerList, () -> "无可分配的档案");

        final Map<Long, List<Customer>> map = customerList.stream().collect(Collectors.groupingBy(Customer::getAdviserId));
        beAssignedList.forEach(c -> {
            int size = Optional.ofNullable(map.get(c.getUserId())).orElse(new ArrayList<>()).size();
            Integer distributableNum = c.getDistributableNum();
            Integer num = c.getNum();
            if (size < distributableNum) {
                c.setDistributableNum(size);
            }
            if (size < num) {
                c.setDistributableNum(size);
                c.setNum(size);
            }
        });

        List<DistributableDTO.BeAssignedDTO> dtoList = beAssignedList.stream().filter(r -> r.getDistributableNum() <= 0 || r.getDistributableNum() < r.getNum()).collect(Collectors.toList());
        BV.isEmpty(dtoList, () -> "分配规则有误，请刷新页面重试");
        return map;
    }


    /**
     * 处理数据
     *
     * @param dto
     * @param map
     * @return 处理完的档案
     */
    private List<Customer> activeDoAllocation(DistributableDTO dto, Map<Long, List<Customer>> map) {
        List<Customer> customerList = new ArrayList<>();
        List<DistributableDTO.BeAssignedDTO> assignedList = dto.getBeAssignedList();
        for (DistributableDTO.BeAssignedDTO assignedDTO : assignedList) {
            customerList.addAll(map.get(assignedDTO.getUserId()).subList(0, assignedDTO.getNum()));
        }
        if (CollectionUtils.isEmpty(customerList)) {
            return new ArrayList<>();
        }
        Long shopId = dto.getShopId();
        Long adviserId = dto.getAdviserId();
        String adviserName = dto.getAdviserName();

        final List<Customer> list = new ArrayList<>();
        final List<StammkundePool> spl = new ArrayList<>();

        for (Customer customer : customerList) {
            customer.setShopId(shopId);
            customer.setAdviserId(adviserId);
            rejectPool(customer.getId(), shopId, adviserName, adviserId, customer.getGroupId());
            spl.add(createPool(customer, adviserName));
            list.add(customer);
        }
        customerService.updateBatchById(list);
        stammkundePoolService.saveBatch(spl);
        return list;
    }

    static class UserInfo {
        private Long userId;
        private String userName;
        private int count;

        public UserInfo() {
        }

        public UserInfo(Long userId, String userName) {
            this.userId = userId;
            this.userName = userName;
        }

        public UserInfo(Long userId, String userName, int count) {
            this.userId = userId;
            this.userName = userName;
            this.count = count;
        }

        public Long getUserId() {
            return userId;
        }

        public void setUserId(Long userId) {
            this.userId = userId;
        }

        public String getUserName() {
            return userName;
        }

        public void setUserName(String userName) {
            this.userName = userName;
        }

        public int getCount() {
            return count;
        }

        public void setCount(int count) {
            this.count = count;
        }

        @Override
        public String toString() {
            return "UserInfo{" +
                    "userId=" + userId +
                    ", userName='" + userName + '\'' +
                    ", count=" + count +
                    '}';
        }
    }
}
