package cn.fw.valhalla.service.data.impl;

import cn.fw.valhalla.dao.mapper.ClueTaskMapper;
import cn.fw.valhalla.domain.db.follow.ClueTask;
import cn.fw.valhalla.domain.dto.FollowPoolDTO;
import cn.fw.valhalla.domain.dto.ShouldBeCompletedDTO;
import cn.fw.valhalla.domain.enums.TaskStateEnum;
import cn.fw.valhalla.domain.query.BeCompletedQuery;
import cn.fw.valhalla.domain.query.FollowPoolQueryVO;
import cn.fw.valhalla.service.data.ClueTaskService;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * 跟进线索任务
 *
 * @author : kurisu
 * @version : 1.0
 * @className : ClueTaskServiceImpl
 * @description : 跟进线索任务
 * @date : 2022-07-22 18:00
 */
@Service
public class ClueTaskServiceImpl extends ServiceImpl<ClueTaskMapper, ClueTask> implements ClueTaskService {
    @Override
    public ClueTask queryOngoingTaskByClueId(Long clueId) {
        return this.getOne(Wrappers.<ClueTask>lambdaQuery()
                        .eq(ClueTask::getClueId, clueId)
                        .eq(ClueTask::getState, TaskStateEnum.ONGOING)
                , Boolean.FALSE);
    }

    @Override
    public List<FollowPoolDTO> followList(FollowPoolQueryVO queryVO) {
        Integer current = queryVO.getCurrent();
        Integer pageSize = queryVO.getPageSize();
        Integer startIndex = (current - 1) * pageSize;
        return Optional.ofNullable(getBaseMapper().followList(startIndex, pageSize, queryVO)).orElse(new ArrayList<>());
    }

    @Override
    public long followListCount(FollowPoolQueryVO queryVO) {
        return Optional.ofNullable(getBaseMapper().followListCount(queryVO)).orElse(0L);
    }

    @Override
    public List<ShouldBeCompletedDTO> queryShouldBeCompleted(BeCompletedQuery query) {
        return Optional.ofNullable(getBaseMapper().queryBeCompleted(query)).orElse(new ArrayList<>());
    }
}
